@@ -2,9 +2,8 @@ use std::collections::HashMap;
 use std::collections::{btree_map::Entry, BTreeMap, BTreeSet};
 
 use crate::analysis::annotation::{Annotation, AnnotationKind};
-use crate::analysis::ast_dependency_detector::ASTDependencyDetector;
+use crate::analysis::ast_dependency_detector::{ASTDependencyDetector, Dependency};
 use crate::analysis::{self, AnalysisPass as REPLAnalysisPass};
-use crate::clarity;
 use crate::clarity::analysis::{types::AnalysisPass, ContractAnalysis};
 use crate::clarity::ast::ContractAST;
 use crate::clarity::contexts::{
@@ -14,8 +13,6 @@ use crate::clarity::contracts::Contract;
 use crate::clarity::costs::{ExecutionCost, LimitedCostTracker};
 use crate::clarity::coverage::TestCoverageReport;
 use crate::clarity::database::{Datastore, NULL_HEADER_DB};
-#[cfg(feature = "cli")]
-use crate::clarity::debug::DebugState;
 use crate::clarity::diagnostic::{Diagnostic, Level};
 use crate::clarity::errors::Error;
 use crate::clarity::events::*;
@@ -25,6 +22,7 @@ use crate::clarity::types::{
     self, PrincipalData, QualifiedContractIdentifier, StandardPrincipalData, Value,
 };
 use crate::clarity::util::StacksAddress;
+use crate::clarity::{self, EvalHook};
 use crate::clarity::{analysis::AnalysisDatabase, database::ClarityBackingStore};
 use crate::clarity::{eval, eval_all};
 use crate::repl::ast;
@@ -32,9 +30,9 @@ use crate::repl::{self, CostSynthesis, ExecutionResult};
 
 pub const BLOCK_LIMIT_MAINNET: ExecutionCost = ExecutionCost {
     write_length: 15_000_000,
-    write_count: 7_750,
+    write_count: 15_000,
     read_length: 100_000_000,
-    read_count: 7_750,
+    read_count: 15_000,
     runtime: 5_000_000_000,
 };
 
@@ -115,8 +113,7 @@ impl ClarityInterpreter {
         snippet: String,
         contract_identifier: QualifiedContractIdentifier,
         cost_track: bool,
-        debug: bool,
-        coverage_reporter: Option<TestCoverageReport>,
+        eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
     ) -> Result<ExecutionResult, Vec<Diagnostic>> {
         let (mut ast, mut diagnostics, success) = self.build_ast(
             contract_identifier.clone(),
@@ -148,8 +145,7 @@ impl ClarityInterpreter {
             snippet,
             analysis,
             cost_track,
-            debug,
-            coverage_reporter,
+            eval_hooks,
         ) {
             Ok(result) => result,
             Err((_, Some(diagnostic), _)) => {
@@ -182,8 +178,7 @@ impl ClarityInterpreter {
         snippet: String,
         contract_identifier: QualifiedContractIdentifier,
         cost_track: bool,
-        debug: bool,
-        coverage_reporter: Option<TestCoverageReport>,
+        eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
     ) -> Result<ExecutionResult, Vec<Diagnostic>> {
         let (annotations, mut diagnostics) = self.collect_annotations(&ast, &snippet);
 
@@ -204,8 +199,7 @@ impl ClarityInterpreter {
             snippet,
             analysis,
             cost_track,
-            debug,
-            coverage_reporter,
+            eval_hooks,
         ) {
             Ok(result) => result,
             Err((_, Some(diagnostic), _)) => {
@@ -237,7 +231,7 @@ impl ClarityInterpreter {
         contract_id: String,
         snippet: String,
         parser_version: u32,
-    ) -> Result<Vec<QualifiedContractIdentifier>, String> {
+    ) -> Result<Vec<Dependency>, String> {
         let contract_id = QualifiedContractIdentifier::parse(&contract_id).unwrap();
         let (ast, _, success) =
             self.build_ast(contract_id.clone(), snippet.clone(), parser_version);
@@ -248,7 +242,19 @@ impl ClarityInterpreter {
         let mut contract_map = HashMap::new();
         contract_map.insert(contract_id.clone(), ast);
         let mut all_dependencies =
-            ASTDependencyDetector::detect_dependencies(&contract_map, &BTreeMap::new());
+            match ASTDependencyDetector::detect_dependencies(&contract_map, &BTreeMap::new()) {
+                Ok(dependencies) => dependencies,
+                Err((_, unresolved)) => {
+                    return Err(format!(
+                        "unresolved dependency(ies): {}",
+                        unresolved
+                            .iter()
+                            .map(|contract_id| contract_id.to_string())
+                            .collect::<Vec<String>>()
+                            .join(",")
+                    ));
+                }
+            };
         let dependencies = match all_dependencies.remove(&contract_id) {
             Some(mut dependencies_set) => dependencies_set.drain().collect(),
             None => vec![],
@@ -403,7 +409,6 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
             let conn = self.datastore.as_clarity_db(&NULL_HEADER_DB);
             let cost_tracker = LimitedCostTracker::new_free();
             let mut global_context = GlobalContext::new(mainnet, conn, cost_tracker);
-            global_context.coverage_reporting = None;
             global_context.begin();
 
             let _ = global_context
@@ -440,8 +445,7 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
         snippet: String,
         contract_analysis: ContractAnalysis,
         cost_track: bool,
-        debug: bool,
-        coverage_reporter: Option<TestCoverageReport>,
+        eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
     ) -> Result<ExecutionResult, (String, Option<Diagnostic>, Option<Error>)> {
         let mut execution_result = ExecutionResult::default();
         let mut contract_saved = false;
@@ -449,7 +453,7 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
         let mut accounts_to_debit = vec![];
         let mut accounts_to_credit = vec![];
         let mut contract_context = ContractContext::new(contract_identifier.clone());
-        let value = {
+        let (value, eval_hooks) = {
             let tx_sender: PrincipalData = self.tx_sender.clone().into();
 
             let mut conn = self.datastore.as_clarity_db(&NULL_HEADER_DB);
@@ -465,11 +469,7 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
                 LimitedCostTracker::new_free()
             };
             let mut global_context = GlobalContext::new(false, conn, cost_tracker);
-            global_context.coverage_reporting = coverage_reporter;
-            #[cfg(feature = "cli")]
-            if debug {
-                global_context.debug_state = Some(DebugState::new(&contract_identifier, &snippet));
-            }
+            global_context.eval_hooks = eval_hooks;
             global_context.begin();
 
             let result = global_context.execute(|g| {
@@ -523,19 +523,21 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
                 }
             });
 
-            execution_result.coverage = global_context.coverage_reporting.take();
             let value = match result {
                 Ok(Some(value)) => value,
                 Ok(None) => Value::none(),
                 Err(e) => {
-                    return Err((
-                        format!(
-                            "Runtime error while interpreting {}: {:?}",
-                            contract_identifier, e
-                        ),
-                        None,
-                        None,
-                    ));
+                    let err = format!(
+                        "Runtime error while interpreting {}: {:?}",
+                        contract_identifier, e
+                    );
+                    if let Some(mut eval_hooks) = global_context.eval_hooks.take() {
+                        for hook in eval_hooks.iter_mut() {
+                            hook.did_complete(Err(err.clone()));
+                        }
+                        global_context.eval_hooks = Some(eval_hooks);
+                    }
+                    return Err((err, None, None));
                 }
             };
 
@@ -695,7 +697,8 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
                     .unwrap();
             }
             global_context.commit().unwrap();
-            value
+
+            (value, global_context.eval_hooks)
         };
 
         execution_result.events = serialized_events;
@@ -710,6 +713,13 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
 
         if !contract_saved {
             execution_result.result = Some(value);
+
+            if let Some(mut eval_hooks) = eval_hooks {
+                for hook in eval_hooks.iter_mut() {
+                    hook.did_complete(Ok(&mut execution_result));
+                }
+            }
+
             return Ok(execution_result);
         }
 
@@ -720,6 +730,12 @@ https://github.com/hirosystems/clarinet/issues/new/choose"#
                 .expect("Unable to save data");
         }
 
+        if let Some(mut eval_hooks) = eval_hooks {
+            for hook in eval_hooks.iter_mut() {
+                hook.did_complete(Ok(&mut execution_result));
+            }
+        }
+
         Ok(execution_result)
     }
 
