@@ -8,6 +8,7 @@ use crate::clarity::types;
 use serde_json::Value;
 use std::collections::BTreeMap;
 
+pub mod ast;
 pub mod interpreter;
 pub mod session;
 pub mod settings;
@@ -15,6 +16,7 @@ pub mod settings;
 pub use interpreter::ClarityInterpreter;
 pub use session::Session;
 pub use settings::SessionSettings;
+pub use settings::{Settings, SettingsFile};
 
 #[derive(Default)]
 pub struct ExecutionResult {
