@@ -1,5 +1,5 @@
 use super::{ClarityInterpreter, ExecutionResult};
-use crate::analysis::ast_dependency_detector::ASTDependencyDetector;
+use crate::analysis::ast_dependency_detector::{ASTDependencyDetector, Dependency};
 use crate::clarity::analysis::ContractAnalysis;
 use crate::clarity::ast::ContractAST;
 use crate::clarity::codec::StacksMessageCodec;
@@ -13,6 +13,7 @@ use crate::clarity::types::{
 };
 use crate::clarity::util::StacksAddress;
 use crate::clarity::variables::NativeVariables;
+use crate::clarity::EvalHook;
 use crate::contracts::{BNS_CONTRACT, COSTS_V1_CONTRACT, COSTS_V2_CONTRACT, POX_CONTRACT};
 use crate::repl::CostSynthesis;
 use crate::{clarity::diagnostic::Diagnostic, repl::settings::InitialContract};
@@ -100,7 +101,7 @@ impl Session {
     fn retrieve_contract(
         &mut self,
         link: &InitialLink,
-    ) -> Result<(String, Vec<QualifiedContractIdentifier>), String> {
+    ) -> Result<(String, Vec<Dependency>), String> {
         let contract_id = &link.contract_id;
         let components: Vec<&str> = contract_id.split('.').collect();
         let contract_deployer = components.first().expect("");
@@ -199,11 +200,14 @@ impl Session {
             if deps.len() > 0 {
                 dependencies.insert(
                     contract_id.to_string(),
-                    deps.clone().into_iter().map(|c| format!("{}", c)).collect(),
+                    deps.clone()
+                        .into_iter()
+                        .map(|c| format!("{}", c.contract_id))
+                        .collect(),
                 );
-                for contract_id in deps.into_iter() {
+                for dep in deps.into_iter() {
                     queue.push_back(InitialLink {
-                        contract_id: contract_id.to_string(),
+                        contract_id: dep.contract_id.to_string(),
                         cache: initial_link.cache.clone(),
                         stacks_node_addr: initial_link.stacks_node_addr.clone(),
                     });
@@ -345,7 +349,7 @@ impl Session {
                     code.to_string(),
                     Some(contract_name.to_string()),
                     true,
-                    false,
+                    None,
                     None,
                 ) {
                     Ok((mut logs, _)) => output.append(&mut logs),
@@ -401,7 +405,19 @@ impl Session {
             }
 
             let dependencies =
-                ASTDependencyDetector::detect_dependencies(&contract_asts, &self.asts);
+                match ASTDependencyDetector::detect_dependencies(&contract_asts, &self.asts) {
+                    Ok(dependencies) => dependencies,
+                    Err((dependencies, unresolved)) => {
+                        for contract_id in unresolved {
+                            output_err.push(format!(
+                                "{}: unresolved dependency: {}",
+                                red!("error"),
+                                contract_id
+                            ));
+                        }
+                        dependencies
+                    }
+                };
             match ASTDependencyDetector::order_contracts(&dependencies) {
                 Ok(ordered_contracts) => {
                     // interpret the contract ASTs in order
@@ -413,7 +429,7 @@ impl Session {
                             ast,
                             contract_identifier.clone(),
                             true,
-                            false,
+                            None,
                             Some("Deployment".into()),
                         ) {
                             Ok((ref mut res_output, result)) => {
@@ -471,7 +487,7 @@ impl Session {
                 POX_CONTRACT.to_string(),
                 Some("pox".to_string()),
                 false,
-                false,
+                None,
                 None,
             )
             .expect("Unable to deploy POX");
@@ -485,7 +501,7 @@ impl Session {
                 BNS_CONTRACT.to_string(),
                 Some("bns".to_string()),
                 false,
-                false,
+                None,
                 None,
             )
             .expect("Unable to deploy BNS");
@@ -499,7 +515,7 @@ impl Session {
                 COSTS_V1_CONTRACT.to_string(),
                 Some("costs-v1".to_string()),
                 false,
-                false,
+                None,
                 None,
             )
             .expect("Unable to deploy COSTS");
@@ -513,7 +529,7 @@ impl Session {
                 COSTS_V2_CONTRACT.to_string(),
                 Some("costs-v2".to_string()),
                 false,
-                false,
+                None,
                 None,
             )
             .expect("Unable to deploy COSTS");
@@ -579,6 +595,8 @@ impl Session {
             cmd if cmd.starts_with("::decode") => self.decode(&mut output, cmd),
             #[cfg(feature = "cli")]
             cmd if cmd.starts_with("::debug") => self.debug(&mut output, cmd),
+            #[cfg(feature = "cli")]
+            cmd if cmd.starts_with("::reload") => self.reload(&mut output),
 
             snippet => {
                 if self.show_costs {
@@ -588,7 +606,7 @@ impl Session {
                         snippet.to_string(),
                         None,
                         true,
-                        false,
+                        None,
                         None,
                     ) {
                         Ok((mut output, result)) => {
@@ -612,7 +630,7 @@ impl Session {
         snippet: String,
         name: Option<String>,
         cost_track: bool,
-        debug: bool,
+        eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
         test_name: Option<String>,
     ) -> Result<(Vec<String>, ExecutionResult), Vec<String>> {
         let light_red = Colour::Red.bold();
@@ -620,8 +638,8 @@ impl Session {
         let result = self.interpret(
             snippet.to_string(),
             name.clone(),
+            eval_hooks,
             cost_track,
-            debug,
             test_name,
         );
         let mut output = Vec::<String>::new();
@@ -656,15 +674,20 @@ impl Session {
 
     #[cfg(feature = "cli")]
     pub fn debug(&mut self, output: &mut Vec<String>, cmd: &str) {
+        use crate::clarity::debug::cli::CLIDebugger;
+
         let snippet = match cmd.split_once(" ") {
             Some((_, snippet)) => snippet,
             _ => return output.push(red!("Usage: ::debug <expr>")),
         };
+
+        let debugger = CLIDebugger::new(&QualifiedContractIdentifier::transient(), snippet);
+
         let mut result = match self.formatted_interpretation(
             snippet.to_string(),
             None,
             true,
-            true,
+            Some(vec![Box::new(debugger)]),
             None,
         ) {
             Ok((mut output, result)) => {
@@ -679,13 +702,59 @@ impl Session {
         output.append(&mut result);
     }
 
+    #[cfg(feature = "cli")]
+    fn reload(&mut self, output: &mut Vec<String>) {
+        self.asts.clear();
+        self.contracts.clear();
+        self.costs_reports.clear();
+        self.coverage_reports.clear();
+        self.initial_contracts_analysis.clear();
+        self.interpreter = ClarityInterpreter::new(
+            self.interpreter.get_tx_sender(),
+            self.settings.repl_settings.clone(),
+        );
+        let contracts = std::mem::take(&mut self.settings.initial_contracts);
+
+        for existing in contracts {
+            let reloaded = match fs::read_to_string(&existing.path) {
+                Ok(code) => code,
+                Err(err) => {
+                    println!(
+                        "{}: Unable to read {}: {}",
+                        red!("error"),
+                        existing.path,
+                        err
+                    );
+                    std::process::exit(1);
+                }
+            };
+
+            self.settings.initial_contracts.push(InitialContract {
+                code: reloaded,
+                path: existing.path,
+                name: existing.name,
+                deployer: existing.deployer,
+            });
+        }
+
+        match self.start() {
+            Ok(_) => {
+                self.get_contracts(output);
+                output.push("Ok, contracts reloaded.".to_string());
+            }
+            Err(e) => {
+                println!("{}", e);
+            }
+        }
+    }
+
     pub fn formatted_interpretation_ast(
         &mut self,
         snippet: String,
         ast: ContractAST,
         contract_identifier: QualifiedContractIdentifier,
         cost_track: bool,
-        debug: bool,
+        eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
         test_name: Option<String>,
     ) -> Result<(Vec<String>, ExecutionResult), Vec<String>> {
         let light_red = Colour::Red.bold();
@@ -694,8 +763,8 @@ impl Session {
             &contract_identifier,
             ast,
             snippet.to_string(),
+            eval_hooks,
             cost_track,
-            debug,
             test_name,
         );
         let mut output = Vec::<String>::new();
@@ -753,7 +822,7 @@ impl Session {
         );
 
         self.set_tx_sender(sender.into());
-        let result = self.interpret(snippet, None, true, false, Some(test_name.clone()))?;
+        let result = self.interpret(snippet, None, None, true, Some(test_name.clone()))?;
         if let Some(ref cost) = result.cost {
             self.costs_reports.push(CostsReport {
                 test_name,
@@ -804,23 +873,24 @@ impl Session {
         contract_identifier: &QualifiedContractIdentifier,
         ast: ContractAST,
         snippet: String,
+        mut eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
         cost_track: bool,
-        debug: bool,
         test_name: Option<String>,
     ) -> Result<ExecutionResult, Vec<Diagnostic>> {
-        let report = if let Some(test_name) = test_name {
+        if let Some(test_name) = test_name {
             let coverage = TestCoverageReport::new(test_name.into());
-            Some(coverage)
-        } else {
-            None
-        };
+            if let Some(hooks) = &mut eval_hooks {
+                hooks.push(Box::new(coverage));
+            } else {
+                eval_hooks = Some(vec![Box::new(coverage)]);
+            };
+        }
         match self.interpreter.run_ast(
             ast,
             snippet,
             contract_identifier.clone(),
             cost_track,
-            debug,
-            report,
+            eval_hooks,
         ) {
             Ok(result) => {
                 if let Some(ref coverage) = result.coverage {
@@ -848,8 +918,8 @@ impl Session {
         &mut self,
         snippet: String,
         name: Option<String>,
+        mut eval_hooks: Option<Vec<Box<dyn EvalHook>>>,
         cost_track: bool,
-        debug: bool,
         test_name: Option<String>,
     ) -> Result<ExecutionResult, Vec<Diagnostic>> {
         let (contract_name, is_tx) = match name {
@@ -858,12 +928,14 @@ impl Session {
         };
         let first_char = contract_name.chars().next().unwrap();
 
-        let report = if let Some(test_name) = test_name {
+        if let Some(test_name) = test_name {
             let coverage = TestCoverageReport::new(test_name.into());
-            Some(coverage)
-        } else {
-            None
-        };
+            if let Some(hooks) = &mut eval_hooks {
+                hooks.push(Box::new(coverage));
+            } else {
+                eval_hooks = Some(vec![Box::new(coverage)]);
+            };
+        }
 
         // Kludge for handling fully qualified contract_id vs sugared syntax
         let contract_identifier = if first_char.to_string() == "S" {
@@ -874,13 +946,10 @@ impl Session {
             QualifiedContractIdentifier::parse(&id).unwrap()
         };
 
-        match self.interpreter.run(
-            snippet,
-            contract_identifier.clone(),
-            cost_track,
-            debug,
-            report,
-        ) {
+        match self
+            .interpreter
+            .run(snippet, contract_identifier.clone(), cost_track, eval_hooks)
+        {
             Ok(result) => {
                 if let Some(ref coverage) = result.coverage {
                     self.coverage_reports.push(coverage.clone());
@@ -973,7 +1042,11 @@ impl Session {
             "{}",
             help_colour
                 .paint("::debug <expr>\t\t\t\tStart an interactive debug session executing <expr>")
-        ))
+        ));
+        output.push(format!(
+            "{}",
+            help_colour.paint("::reload \t\t\t\tReload the existing contract(s) in the session")
+        ));
     }
 
     fn parse_and_advance_chain_tip(&mut self, output: &mut Vec<String>, command: &str) {
@@ -1062,7 +1135,7 @@ impl Session {
             _ => return output.push(red!("Usage: ::encode <expr>")),
         };
 
-        let result = self.interpret(snippet.to_string(), None, false, false, None);
+        let result = self.interpret(snippet.to_string(), None, None, false, None);
         let value = match result {
             Ok(result) => {
                 let mut tx_bytes = vec![];
@@ -1112,7 +1185,7 @@ impl Session {
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
         let (mut result, cost) =
-            match self.formatted_interpretation(snippet, None, true, false, None) {
+            match self.formatted_interpretation(snippet, None, true, None, None) {
                 Ok((output, result)) => (output, result.cost.clone()),
                 Err(output) => (output, None),
             };
@@ -1223,7 +1296,7 @@ impl Session {
     pub fn get_costs(&mut self, output: &mut Vec<String>, cmd: &str) {
         let snippet = cmd.to_string().split_off("::get_costs ".len());
         let (mut result, cost) =
-            match self.formatted_interpretation(snippet, None, true, false, None) {
+            match self.formatted_interpretation(snippet, None, true, None, None) {
                 Ok((output, result)) => (output, result.cost.clone()),
                 Err(output) => (output, None),
             };
